
def printTable(table: []):
    '''
    表格化输出信息
    
    - 参数：
        - table: [],数组里请保存dict类型数据
    
    - 返回值:
        str
    '''
    if table:
        m_str=''
        keyheader = []
        tableWidth=[]
        for name in table[0].keys():
            keyheader.append(name)
        offset=0
        for item in table:
            for name,value in item.items():
                if offset < len(tableWidth):
                    if len(name)> tableWidth[offset]:
                        tableWidth[offset]=len(name)
                else:
                    tableWidth.append(len(name))
                if len(str(value))>tableWidth[offset]:
                    tableWidth[offset]= len(str(value))
                offset+=1
            offset=0
        offset = 0
        for header in keyheader:
            if offset == 0:
                m_str+="+"
            for i in range(0, tableWidth[offset]+10):
                m_str+="-"
            m_str+='+'
            offset += 1
        offset = 0
        m_str+='\r\n'
        for header in keyheader:
            m_str+='|'
            m_str+=' '+header
            for i in range(0, tableWidth[offset]+10-len(header)-1):
                m_str+=' '
            offset+=1
        m_str+='|'
        m_str+='\r\n'
        offset = 0
        for header in keyheader:
            if offset == 0:
                m_str+="+"
            for i in range(0, tableWidth[offset]+10):
                m_str+="-"
            offset+=1
            m_str+='+'
        m_str+='\r\n'
        offset=0
        for item in table:
            for name, value in item.items():
                m_str+='|'
                m_str+=' '+str(value)
                for i in range(0, tableWidth[offset]+10-len(str(value))-1):
                    m_str+=' '
                offset += 1
            m_str+='|'
            m_str+='\r\n'
            offset = 0
        offset=0
        for header in keyheader:
            if offset == 0:
                m_str+="+"
            for i in range(0, tableWidth[offset]+10):
                m_str+="-"
            offset+=1
            m_str+='+'
        m_str+='\r\n'
        return m_str
    return ''


import json
import os
import logging
from rest_framework.request import Request
from TDhelper.network.http.REST_HTTP import GET, POST, PUT, DELETE, serializePostData, ContentType
from TDhelper.generic.recursion import recursion, recursionCall
from TDhelper.generic.randGetValue import getValue

class permission:
    def __init__(self, host, platformKey, secret, httpHeaders={}, token_key='api-token') -> None:
        self._host = getValue(host).rstrip("/") + "/api/"
        self._platformKey = platformKey
        self._secret = secret
        self._httpHeaders = httpHeaders
        self._httpHeaders[token_key] = self._secret

    def register(self, path, encode='utf8') -> None:
        if os.path.exists(path):
            conf = ''
            with open(path, "r", -1, encoding=encode) as f:
                conf = f.read()
                f.close()
            if conf:
                conf = json.loads(conf)
                recursionCall(self._register_permission, 200, conf, **{})
        else:
            raise Exception("Not Found '%s'" % path)

    def _register_permission(self, config, **kwargs):
        if self._platformKey:
            config["permission_key"] = (
                self._platformKey + "." + config["permission_key"]
            )
        if not self._host.endswith("/"):
            self._host += "/"
        checkState, msg = self.checkAttr(
            ["permission_name", "permission_key", "permission_domain", "permission_uri","permission_enable","permission_attribute"], config)
        if not checkState:
            raise Exception("can not found attribute '%s'" % msg)
        post_data = {
            "permission_name": config["permission_name"],
            "permission_key": config["permission_key"],
            "permission_domain": config["permission_domain"],
            "permission_uri": config["permission_uri"],
            "permission_enable": config["permission_enable"],
            "permission_parent": config["permission_parent"]
            if "permission_parent" in config
            else kwargs["permission_parent"]
            if "permission_parent" in kwargs
            else 0,
            "permission_attribute": config["permission_attribute"] if config["permission_attribute"] else 0
        }
        state, body = POST(
            uri=self._host + "permissions/",
            post_data=post_data,
            http_headers=self._httpHeaders,
            content_type=ContentType.JSON,
            time_out=15,
        )
        m_parent_id = 0
        if state == 200:
            m_ret = str(body, encoding="utf-8")
            m_ret_json = json.loads(m_ret)  # , encoding="utf-8")
            if m_ret_json["state"] == 200:
                m_parent_id = m_ret_json["msg"]["permission_id"]
                logging.info(
                    "create permission '%s' success." % config["permission_name"]
                )
            else:
                logging.info(
                    "create permission '%s' failed.error(%s)"
                    % (config["permission_name"], m_ret_json["msg"])
                )
        else:
            logging.info("create permission '%s' failed.error(%s)" %
                         (config["permission_name"], body))
        if "children" not in config:
            kwargs["break"] = False
        else:
            for item in config["children"]:
                kwargs["permission_parent"] = m_parent_id
                recursionCall(
                    self._register_permission, kwargs["limit"], item, **kwargs
                )
            kwargs["break"] = False
        return config, kwargs

    def checkAttr(self, attrs: list, rel: dict):
        state = True
        msg = ""
        for o in attrs:
            if o not in rel:
                state = False
                msg = "".join([o, ","])
        return state, msg


class PermissionACL:
    '''权限检查
    args:
        rpc_key: permission key.
        permission_rpc_service: permission service key.
        method: permission service validate method.
        params_container_class: context class.
        platformKey: service platform key.
        tokenKey: token key filed.
    '''

    def __init__(
        self, permission_rpc_service, method, params_container_class, platformKey=None, tokenKey="permission-token"
    ):
        self._params_container = params_container_class
        self._platformKey = platformKey
        self._tokenKey = tokenKey
        self._rpc = None
        self._rpc_service_key= permission_rpc_service
        self._rpc_service_method = method

    def addRPCHandle(self, handle):
        self._rpc = handle

    def _core(self, premissionKey=None, debug=False, *args, **kwargs):
        validate_state = False
        if not debug:
            if self._platformKey:
                self._platformKey += "."
                self._platformKey = self._platformKey.upper()
            eventKey = self._platformKey + premissionKey.upper()
            params_instance = None
            for k in args:
                if isinstance(k, self._params_container):
                    params_instance = k
                    break
            if not params_instance:
                for k, v in kwargs:
                    if isinstance(v, self._params_container):
                        params_instance = v
                        break
            if not params_instance:
                return 500,"can found context."
            if isinstance(params_instance, Request):
                if self._tokenKey in params_instance._request.headers:
                    token = params_instance._request.headers[self._tokenKey]
                else:
                    return 500,"http headers can not found '%s' key." % self._tokenKey
            elif isinstance(params_instance, dict):
                token = params_instance[self._tokenKey]
            elif isinstance(params_instance, (int, str, float)):
                token = params_instance
            else:
                token = getattr(params_instance, self._tokenKey)
            if token:
                if self._rpc:
                    validate_ret = self._rpc.__call__(self._rpc_service_key,self._rpc_service_method, **{"token": token, "event": eventKey})
                    if validate_ret:
                        if validate_ret["state"] == 200:
                            validate_state = True
                        else:
                            return 500,"access error.(%s)" % validate_ret["msg"]
                    else:
                        return 500,"access http error. remote call error."
                else:
                    return 500,"rpc handle is none."
            else:
                return 500,"'%s' is None."%self._tokenKey
        else:
            validate_state = True
        if validate_state:
            return 200,"success"
        else:
            return 401,"Unauthorized"



from TDhelper.robot.people import action, actionMode
from TDhelper.robot.struct.base import joint
from TDhelper.robot.struct.toe import TOE
from TDhelper.robot.struct.ankle import ANKLE
from TDhelper.robot.struct.knee import KNEE
from TDhelper.robot.struct.hip import HIP
from TDhelper.robot.control.D_33890 import device_D_33890


class LEGS(action):
    def __init__(self):
        super(LEGS, self).__init__()
        self.leftTOE = []
        self.rightTOE = []
        self.leftTOE = self._generateToes(label="左")
        self.rightTOE = self._generateToes(label="右")
        self.leftANKLE = self._generateAnkles(label="左")
        self.rightANKLE = self._generateAnkles(label="右")
        self.leftKNEE = self._generateKnees(label="左")
        self.rightKNEE = self._generateKnees(label="右")
        self.leftHIP = self._generateHips(label="左")
        self.rightHIP = self._generateHips(label="右")

    def _generateHips(self, label: str = ''):
        '''
            生成胯关节
        '''
        m_joints = []
        power = {
            'updown': device_D_33890("%s胯关节上下舵机" % label, (0, 180), 90),
            'leftright': device_D_33890("%s胯关节左右舵机" % label, (0, 180), 90)
        }
        m_joints.append(joint('%s胯' % label, powerSystem=power))
        return HIP(m_joints)

    def _generateKnees(self, label: str = ""):
        '''
            生成膝盖关节
        '''
        m_joints = []
        power = {
            'updown': device_D_33890("%s膝盖关节上下舵机" % label, (0, 140), 0)
        }
        m_joints.append(joint('%s膝盖' % label, powerSystem=power))
        return KNEE(m_joints)

    def _generateAnkles(self, label: str = ""):
        '''
            生成脚踝
        '''
        m_joints = []
        power = {
            'updown': device_D_33890("%s踝关节上下舵机" % label, (0, 90), 0),
            'leftright': device_D_33890("%s踝关节左右舵机" % label, (0, 90), 45)
        }
        m_joints.append(joint('%s脚踝' % label, powerSystem=power))
        return ANKLE(m_joints)

    def _generateToes(self, label: str = ''):
        '''
            生成脚趾
        '''
        ret = [None, None, None, None, None]
        m_joints = []
        # 大脚趾
        power = {
            'updown': device_D_33890("%s脚大拇趾第一关节" % label, (0,90),0)
        }
        m_joints.append(joint('%s脚大拇指第一关节' % label, powerSystem=power))
        power = {
            'updown': device_D_33890("%s脚大拇趾第二关节" % label, (-90,0),0)
        }
        m_joints.append(joint('%s脚大拇指第二关节' % label, powerSystem=power))
        ret[0] = TOE(m_joints)
        # 二号脚趾
        m_joints.clear()
        power = {
            'updown': device_D_33890("%s脚第二脚趾指第一关节" % label,(0,90),0)
        }
        m_joints.append(joint('%s脚第二脚趾指第一关节' % label, powerSystem=power))
        power = {
            'updown': device_D_33890("%s脚第二脚趾指第二关节" % label, (-90,0),0)
        }
        m_joints.append(joint('%s脚第二脚趾指第二关节' % label, powerSystem=power))
        power = {
            'updown': device_D_33890("%s脚第二脚趾指第三关节" % label, (-90,0),0)
        }
        m_joints.append(joint('%s脚第二脚趾指第三关节' % label, powerSystem=power))
        ret[1] = TOE(m_joints)
        # 三号脚趾
        m_joints.clear()
        power = {
            'updown': device_D_33890("%s脚第三脚趾指第一关节" % label, (0,90),0)
        }
        m_joints.append(joint('%s脚第三脚趾指第一关节' % label, powerSystem=power))
        power = {
            'updown': device_D_33890("%s脚第三脚趾指第二关节" % label, (-90,0),0)
        }
        m_joints.append(joint('%s脚第三脚趾指第二关节' % label, powerSystem=power))
        power = {
            'updown': device_D_33890("%s脚第三脚趾指第三关节" % label, (-90,0),0)
        }
        m_joints.append(joint('%s脚第三脚趾指第三关节' % label, powerSystem=power))
        ret[2] = TOE(m_joints)
        # 四号脚趾
        m_joints.clear()
        power = {
            'updown': device_D_33890("%s脚第四脚趾指第一关节" % label, (0,90),0)
        }
        m_joints.append(joint('%s脚第四脚趾指第一关节' % label, powerSystem=power))
        power = {
            'updown': device_D_33890("%s脚第四脚趾指第二关节" % label, (-90,0),0)
        }
        m_joints.append(joint('%s脚第四脚趾指第二关节' % label, powerSystem=power))
        power = {
            'updown': device_D_33890("%s脚第四脚趾指第三关节" % label,  (-90,0),0)
        }
        m_joints.append(joint('%s脚第四脚趾指第三关节' % label, powerSystem=power))
        ret[3] = TOE(m_joints)
        # 五号脚趾
        m_joints.clear()
        power = {
            'updown': device_D_33890("%s脚第五脚趾指第一关节" % label, (0,90),0)
        }
        m_joints.append(joint('%s脚第五脚趾指第一关节' % label, powerSystem=power))
        power = {
            'updown': device_D_33890("%s脚第五脚趾指第三关节" % label, (-90,0),0)
        }
        m_joints.append(joint('%s脚第五脚趾指第二关节' % label, powerSystem=power))
        power = {
            'updown': device_D_33890("%s脚第五脚趾指第三关节" % label, (-90,0),0)
        }
        m_joints.append(joint('%s脚第五脚趾指第三关节' % label, powerSystem=power))
        ret[4] = TOE(m_joints)
        return ret



from enum import Enum

class STATUS(Enum):
    '''
        HTTP状态
    '''
    HTTP_SUCCESS= 200
    HTTP_BAD_REQUEST= 400
    HTTP_NOT_FOUND= 404
    HTTP_REQUEST_TIME= 408
    HTTP_UNAVAILABLE_FOR_LEGAL_REASONS= 451
    HTTP_INTERNAL_SERVER_ERROR= 500
    HTTP_BAD_GATEWAY=502
    HTTP_SERVICE_UNAVAILABLE= 503
    HTTP_GATEWAY_TIME_OUT= 504

    '''
        REQUEST对象错误
    '''
    REQUEST_CONNECT_ERROR= 10001
    REQUEST_REQUEST_RECONNECT_EXCEED_LIMIT_MAX= 10002
    REQUEST_ERROR= 10003
    REQUEST_URL_IS_NONE= 10004
    REQUEST_IS_NONE= 10005
    '''
        SPIDER错误
    '''

    SPIDER_SUCCESS= 20000
    # 配置文件未加载
    SPIDER_CONFIG_IS_NOT_LOAD= 20001
    # 找不到配置文件对应的KEY
    SPIDER_CONFIG_CAN_NOT_FOUND_KEY= 20002
    # 无法获取页面类型
    SPIDER_PAGE_TYPE_NOT_FOUND= 20003
    # 内容未变更
    SPIDER_FINGERPRINT_IS_REPEAT= 20004
    # 按规则未匹配到内容
    SPIDER_REGAULE_MATCH_IS_ERROR= 20005
    # 截取内容错误
    SPIDER_INTERCEPT_ERROR= 20006
    # 找不到页面的分析规则
    SPIDER_PAGE_ANALYSIS_RULES_CAN_NOT_FOUND= 20007
    # 页面过滤出出错
    SPIDER_PAGE_FILTER_ERROR= 20008
    # 页面分析过滤出错
    SPIDER_PAGE_ANALYSIS_FILTER_FAIL= 20009
    # 页面分析ACTION错误
    SPIDER_PAGE_ANALYSIS_ACTIONS_ERROR= 20010
    # 页面分析失败
    SPIDER_PAGE_ANALYSIS_ACTION_FAIL= 20011
    # 序列化数据出错
    SPIDER_SERIALIZE_ERROR= 20012
    # 序列化数据校验失败
    SPIDER_SERIALIZE_DATA_VERIFY_FAIL= 20013
    # 跳过爬取
    SPIDER_EXCLUDE= 20014

    #CONFIG 状态
    CONFIG_SUCCESS= 30000
    CONFIG_ERROR= 30001
    # 错误
    SPIDER_ERROR= -1


from TDhelper.document.ini.model import model
from TDhelper.document.ini.fields import FieldType

class t(model):
    a= FieldType(str,"SERVICE.host")
    b= FieldType(int,"SERVICE.name")

if __name__ == "__main__":
    a= t(r"C:\Home\dev\Python\pypi\src\TDhelper\document\ini\1.ini")
    print(a.a)
    print(a.b)


#!/usr/bin/env python3.6
# -*- encoding: utf-8 -*-
'''
@File    :   dictHelper.py
@Time    :   2020/09/24 07:13:30
@Author  :   Tang Jing
@Version :   1.0.0
@Contact :   yeihizhi@163.com
@License :   (C)Copyright 2020
@Desc    :   None
'''

# here put the import lib

# code start


def createDictbyStr(key: str, source={}, value: any = None):
    ''' [obsolete]createDictbyStr
        (obsolete)create an dict object by formatter string. this method has obsoleted, can use appendDict method.
        - Params:
        -   key: <str>, the string that generates the dict.
        -   source: <dict>, the targte dict, default {}.
        -   value: <any>, set the value of the new dict last child node.
        - Returns: <dict>, the new dict.
    '''
    result = source
    node = ''
    total = 1
    m_source = result
    for k in key.split('.'):
        total += 1
        node += ("." if node else "") + k
        if node:
            search_count = 1
            m_m_source = m_source
            for k1 in node.split('.'):
                search_count += 1
                if search_count > len(node.split('.')):
                    if k1 not in m_m_source:
                        m_m_source.update({k1: value})
                    else:
                        m_m_source = m_m_source[k1]
                else:
                    if k1 not in m_m_source:
                        m_m_source.update({k1: {}})
                    else:
                        if not isinstance(m_m_source[k1], dict):
                            m_m_source[k1] = {}
                    m_m_source = m_m_source[k1]
    return result


def appendDict(k, v, source: 'dict'):
    k = k.split('.')
    count = 0
    instance_dict = source
    for item in k:
        count += 1
        if count == len(k):
            instance_dict.update({item: v})
        else:
            if item not in instance_dict:
                instance_dict.update({item: {}})
            instance_dict = instance_dict[item]


def findInDict(findKey, source: 'dict|type'):
    '''
        根据finkey查找dict.
        - Params:
        -   findkey: <str>, formatter("node1.node2.node3").
        -   source: <dict>, it is your search target, findkey must in it. if findkey not in it, then raise an error
        - Returns: <any>, if can't found, then return None.
    '''
    result = None
    search_node = ''
    for key in findKey.split('.'):
        search_node += ("." if search_node else "") + key
        if isinstance(source, dict):
            if key not in source:
                raise Exception("can not found key(%s)" % search_node)
            result = source[key]
            source = source[key]
        elif isinstance(source, object):
            result = getattr(source, key)
            if not result:
                raise Exception("can not found key(%s)" % search_node)
            else:
                source = result
    return result


def hitDict(k, source: "dict"):
    search_node = ''
    instace_dict = source
    keys = k.split('.')
    count = 0
    for key in keys:
        count += 1
        search_node += ("." if search_node else "") + key
        if key not in instace_dict:
            raise Exception("can not found key(%s)" % search_node)
        if count == len(keys):
            return instace_dict[key]
        else:
            instace_dict = instace_dict[key]


def setDictValue(k, v, source: 'dict|type'):
    search_node = ''
    instace_dict = source
    keys = k.split('.')
    count = 0
    for key in keys:
        count += 1
        search_node += ("." if search_node else "") + key
        if isinstance(instace_dict, dict):
            if key not in instace_dict:
                raise Exception("can not found key(%s)" % search_node)
            if count == len(keys):
                instace_dict[key] = v
            else:
                instace_dict = instace_dict[key]
        elif isinstance(instace_dict, object):
            result = getattr(instace_dict, key)
            if not result:
                raise Exception("can not found key(%s)" % search_node)
            if count == len(keys):
                setattr(instace_dict, v)
            else:
                instace_dict = result


def destoryKey(k, source: "dict"):
    search_node = ''
    instace_dict = source
    keys = k.split('.')
    count = 0
    for key in keys:
        count += 1
        search_node += ("." if search_node else "") + key
        if key not in instace_dict:
            raise Exception("can not found key(%s)" % search_node)
        if count == len(keys):
            instace_dict.pop(key)
        else:
            instace_dict = instace_dict[key]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__autho__ = "Tony.Don"
__lastupdatetime__ = "2017/09/20"

import requests
from requests import exceptions
'''
class
    TDhelper.generic.m_http
description
    http帮助类
'''


class m_http:
    context = None
    res = None
    currentEncoding= ''
    def __init__(self):
        self.context = requests
    def autoTransformEncoding(self,text):
        if text:
            if self.res and self.res.encoding=="ISO-8859-1":
                encodings= requests.utils.get_encodings_from_content(text)
                if encodings:
                    encoding= encodings[0]
                else:
                    encoding= self.res.apparent_encoding
                self.currentEncoding= encoding
                text= self.res.content.decode(encoding,'replace')
        return text

    def post(self, url, header: dict= None, data= None, timeout= 5, encoding='auto'):
        try:
            if header:
                self.res = self.context.post(
                    url=url, data=data, headers=header, timeout=timeout)
            else:
                self.res = self.context.post(url, timeout=timeout)
            if self.res:
                if encoding.lower()=='auto':
                    return self.autoTransformEncoding(self.res.text), self.res.status_code
                else:
                    self.res.encoding= encoding
                    self.currentEncoding= encoding
                    return self.res.text, self.res.status_code
            return url, "REQUESTS_OBJECT_IS_NULL"
        except exceptions.HTTPError as e:
            return e, "CONNECT_IS_ERROR"
        except exceptions.Timeout as e:
            return e, "TIME_OUT"
        except exceptions.ConnectTimeout as e:
            return e, 'CONNECT_TIME_TIMEOUT'
        except exceptions.ProxyError as e:
            return e, 'PROXY_ERROR'
        except exceptions.SSLError as e:
            return e, 'SSL_ERROR'
        except exceptions.URLRequired as e:
            return e, 'URL_REQUIRED'
        except exceptions.BaseHTTPError as e:
            return e, 'BASE_HTTP_ERROR'
        except Exception as e:
            return e, "ERROR"

    # url:wetsite url address
    def getcontent(self, p_url, p_timeout=5, headers: dict = None, encoding='auto'):
        '''
        Featuren\r\n
            getcontent(self,url)\r\n
        Description\r\n
            获取url内容\r\n
        Args\r\n
            url\r\n
                type:string\r\n
                description:目标URL\r\n
        '''
        if self.context:
            try:
                if not headers:
                    self.res = self.context.get(url=p_url, timeout=p_timeout)
                else:
                    self.res = self.context.get(
                        url=p_url, timeout=p_timeout, headers=headers)
                if self.res:
                    if encoding.lower()=='auto':
                        return self.autoTransformEncoding(self.res.text), self.res.status_code
                    else:
                        self.res.encoding= encoding
                        self.currentEncoding= encoding
                        return self.res.text, self.res.status_code
                return p_url, "REQUESTS_OBJECT_IS_NULL"
            except exceptions.HTTPError as e:
                return e, "CONNECT_IS_ERROR"
            except exceptions.Timeout as e:
                return e, "TIME_OUT"
            except exceptions.ConnectTimeout as e:
                return e, 'CONNECT_TIME_TIMEOUT'
            except exceptions.ProxyError as e:
                return e, 'PROXY_ERROR'
            except exceptions.SSLError as e:
                return e, 'SSL_ERROR'
            except exceptions.URLRequired as e:
                return e, 'URL_REQUIRED'
            except exceptions.BaseHTTPError as e:
                return e, 'BASE_HTTP_ERROR'
            except Exception as e:
                return e, "ERROR"

    def download(self, p_url, p_timeout=5, headers: dict = None):
        if self.context:
            try:
                if not headers:
                    self.res = self.context.get(url=p_url, timeout=p_timeout)
                else:
                    self.res = self.context.get(
                        url=p_url, timeout=p_timeout, headers=headers)
                if self.res:
                    return self.res.content, self.res.status_code
                return p_url, "REQUESTS_OBJECT_IS_NULL"
            except Exception as e:
                return e, "CONNECT_IS_ERROR"

    def https(self, url, *args, **kwargs):
        '''
        certificate
        '''
        pass



class validate_permission:

    def __init__(
        self, rq_class, platformKey
    ):

        self.__error_message__ = ""
        self.__state_code__ = -1
        self.__handle_state__ = False
        self.__debug_state__ = False
        self.__rpc_service_method__ = None

        self.__request_context_class__ = rq_class
        self.__platform_key__ = platformKey

    @property
    def error_message(self):
        return self.__error_message__

    def __reset_error_cfg__(self):
        self.__error_message__ = ""
        self.__state_code__ = -1

    def __error_update__(self, msg: str | list, code=-1):
        join_str = "" if not self.__error_message__ else ";\r\n"
        msg = (lambda v: v if isinstance(v, list) else [v])(msg)
        if self.__error_message__:
            msg.insert(0, self.__error_message__)
        self.__error_message__ = join_str.join(msg)
        self.__state_code__ = code

    def set_permission_handle(self, handle, permission_service_key, func_name):
        '''set permission client rpc handle.
        
        params:

            handle - <class: TDhelper.network.rpc.client> : rpc handle.
            
            permission_service_key - <class: string> : rpc service key.
            
            func_name - <class: string> : validate permission func key.
            
        returns:

            None.
        '''
        if handle:
            o = handle.__get_service__(permission_service_key)
            if o:
                m_method = o.__get_method_handle__(func_name)
                if m_method:
                    self.__rpc_service_method__ = m_method
                    self.__handle_state__= True
                else:
                    raise Exception("not found method %s in %s." %
                                    (func_name, permission_service_key))
            else:
                raise Exception("not found service %s in handle" % permission_service_key)
        else:
            raise Exception("handle is none.")

    def validate(self, group= 0, permission_key=None):
        '''validate permission.
        
        params:

            group - <class: int> : group id. 
            
            permission_key - <class: str> : permission key string.
            
        returns:

            object - <class: tuple> : 2 length. index 0 is state, index 1 is message. 
            
            example : (200, "success")
        
        '''
        if self.__handle_state__:
            if not self.__debug_state__:
                m_permission_key = ("." if self.__platform_key__ else "").join(
                    [self.__platform_key__, permission_key.upper()])
                validate_ret = self.__rpc_service_method__(
                        **{"g":group,"p": m_permission_key})
                if validate_ret:
                        if validate_ret["state"] == 200:
                            return 200, "success"
                        else:
                            return 500, "access error(%s). " % validate_ret["msg"]
                else:
                        return 500, "access http error. remote call error."
            else:
                return 200, "debug mode, no checked permission."
        else:
            return 500, self.error_message
                



from TDhelper.document.ini.meta import iniMeta
import os


class model(metaclass=iniMeta):
    def __init__(self, source):
        assert isinstance(source, str), "source is not str type."
        if os.path.exists(source):
            self.Meta.source = source if isinstance(source, str) else [source]
            self.__instance__()
        else:
            raise "can't found file:'%s'" % (source)

    class Meta:
        source = None